from intelligence.base import *
import requests
import json
import base64


URL = "https://www.filescan.io/api/reports/search"


class FileScanIO(OnlineChecker):

    name = "FileScanIO"
    
    def check(self, analysis):
        detections = { }
        key = self.options.get("key")
        if not key:
            raise KeyError("No API key")
        message = ""
        session = requests.Session( )
        session.headers.update({'accept': 'application/json'})
        session.headers.update({'X-Api-Key': key})
        session.headers.update({'User-Agent': 'malcat'})

        response = session.get(URL, params={
            "sha256": analysis.entropy.sha256,
            })
        if response.ok:
            data = response.json()
            if "message" in data:
                raise ValueError(message)
            #print(json.dumps(data, indent=4))
            items = data.get("items", [])
            if not items:
                return None
            for item in items:
                filename = item.get("file", {}).get("name", "???")
                verdict = item.get("verdict")
                if verdict is not None:
                    level = DetectionLevel.from_text(verdict)
                else:
                    level = DetectionLevel.UNKNOWN
                name = "+".join([tag.get("tag", {}).get("name", "???") for tag in item.get("tags", [])])
                detections[filename] = OnlineDetection(level, name)
        elif response.status_code == 404:
            return None
        else:
            raise response.raise_for_status()

        url = "https://www.filescan.io/search-result?query={}".format(base64.b64encode(analysis.entropy.sha256.encode("ascii")).decode("ascii"))
        return OnlineResult(detections=detections, 
            url=url
        )

